library cobefile;

import 'dart:convert';
import 'package:crypto/crypto.dart';
import 'package:http_parser/http_parser.dart';
import 'package:http/http.dart' as http;
import 'package:uuid/uuid.dart';

const String COBEFILE_URL =
    'http://cobefile-env.eba-rgu2s6df.us-east-2.elasticbeanstalk.com/';
/*const String COBEFILE_URL =
    'https://localhost:44342/';*/

enum FileTypeCobe { image, video }

class CobeFile {
  final String appId;
  final String apiKey;

  CobeFile(this.appId, this.apiKey);

  Future<Map<String, dynamic>> upload(
      String filePath, String type, FileTypeCobe fileType) async {
    var contentType;
    switch (fileType) {
      case FileTypeCobe.image:
        contentType = MediaType('image', 'jpeg');
        break;
      case FileTypeCobe.video:
        contentType = MediaType('video', 'mp4');
        break;
      default:
        contentType = MediaType('image', 'jpeg');
        break;
    }

    var request = http.MultipartRequest('POST', Uri.parse(COBEFILE_URL));

    request.files.add(await http.MultipartFile.fromPath('files', filePath,
        contentType: contentType));
    request.fields['type'] = type;

    request.headers['Access-Control-Allow-Origin'] = '*';
    request.headers['Authorization'] = 'amx ' + _sign();
    request.headers['Cache-Control'] = 'no-cache';

    var response = await request.send();

    var responseBody = await response.stream.bytesToString();
    return json.decode(responseBody);
  }

  String _sign() {
    String timestamp = DateTime.now().toUtc().toIso8601String();
    String uuidv4Complete = Uuid().v4();
    String nonce = uuidv4Complete.substring(0, 23);
    nonce = nonce.replaceRange(14, 16, 'cb');
    nonce = nonce.replaceRange(19, 21, 'js');

    String signatureRawData = appId + 'POSTcobefile' + timestamp + nonce;

    var secretKeyByteArray = base64.decode(apiKey);
    var signature = utf8.encode(signatureRawData);

    var hmac = Hmac(sha256, secretKeyByteArray);
    Digest digest = hmac.convert(signature);
    String requestSignature = base64.encode(digest.bytes);

    //print('signatureRawData: ' + signatureRawData);
    //print('requestSignature: ' + requestSignature);

    return appId + '|' + requestSignature + '|' + nonce + '|' + timestamp;
  }
}
